import $ from 'jquery'
import { Plugin } from 'foundation-sites/js/foundation.core.plugin'
import { LocalStore } from './storeWrapper'

/**
 * CookieBar module.
 *
 * @module foundation.cookieBar
 */

class CookieBar extends Plugin {
  /**
   * Creates a new instance.
   * @class
   * @name CookieBar
   * @fires CookieBar#init
   * @param {JQuery} element - jQuery object representing the CookieBar.
   * @param {object} options - Overrides to the default plugin settings.
   */
  _setup (element, options = {}) {
    this.$element = element
    this.className = 'CookieBar'
    // defaults, fallbacks
    this.options = $.extend(true, {}, CookieBar.defaults, this.$element.data(), options)
    this.store = new LocalStore(this.options.storeName)

    // Trigger previously accepted events or initialize the cookie bar.
    const preferences = this.store.read()
    if (preferences) {
      this._dispatchEvents(preferences)
    } else if (preferences === null) {
      this._init()
    }

    this._log('setup', this)
  }

  /**
   * Initializes the cookiebar: registers click handlers and displays the cookie bar.
   * @function
   * @private
   */
  _init () {
    this._log('init', this)

    // Click handler for accepting cookies.
    $(`.${this.options.acceptClass}, [data-accept]`, this.$element).click((e) => {
      const events = (e.target.dataset.accept || this.options.defaultEvent).split(' ')
      this.accept(events)
    })
    // Click handler for blocking cookies.
    $(`.${this.options.blockClass}, [data-block]`, this.$element).click(() => { this.block() })

    // Prepare display.
    if (typeof this.options.init === 'function') {
      this.options.init(this.$element)
    } else {
      this.$element.css('bottom', -1 * this.$element.outerHeight())
    }

    this.show()
  }

  /**
   * Accept tracking events.
   * @function
   * @param {Array.string} events - Accepted events.
   * @fires cookieBar#accepted
   */
  accept (events) {
    this._log('accept', events)
    /**
     * Fires when cookies have been accepted.
     * @event cookieBar#blocked
     */
    this.$element.trigger('accepted.zf.cookieBar', [events])

    this.hide()
    this._dispatchEvents(events)
    this.store.write(events)
  }

  /**
   * Block tracking events.
   * @function
   * @fires cookieBar#blocked
   */
  block () {
    this._log('block all')
    /**
     * Fires when cookies have been blocked.
     * @event cookieBar#blocked
     */
    this.$element.trigger('blocked.zf.cookieBar')

    this.hide()
    this.store.write([])
  }

  /**
   * Reveal the sticky button.
   * @function
   * @fires cookieBar#show
   * @fires cookieBar#shown
   */
  show () {
    this._log('show', this.$element)
    /**
     * Fires when the showing starts.
     * @event cookieBar#show
     */
    this.$element.trigger('show.zf.cookieBar')

    if (typeof this.options.show === 'function') {
      this.options.show(this.$element)
    } else {
      this.$element.show()
      this.$element.css('bottom', '0').css('opacity', 1)
    }

    /**
     * Fires when showing is finished.
     * @event cookieBar#shown
     */
    this.$element.trigger('shown.zf.cookieBar')
  }

  /**
   * Hide the sticky button.
   * @function
   * @fires cookieBar#hide
   * @fires cookieBar#hidden
   */
  hide () {
    this._log('hide', this.$element)
    /**
     * Fires when the hiding starts.
     * @event cookieBar#hide
     */
    this.$element.trigger('hide.zf.cookieBar')

    if (typeof this.options.hide === 'function') {
      this.options.hide(this.$element)
    } else {
      this.$element.css('bottom', -1 * this.$element.outerHeight()).css('opacity', 0)
      if (this.$element.css('transition-duration')) {
        this.$element.on('transitionend.zf.cookiebar', () => {
          this.$element.hide().off('transitionend.zf.cookiebar')
        })
      } else {
        this.$element.hide()
      }
    }

    /**
     * Fires when the hiding is finished.
     * @event cookieBar#hidden
     */
    this.$element.trigger('hidden.zf.cookieBar')
  }

  /**
   * Dispatch tracking events.
   * @function
   * @param {Array.string} events - Events.
   * @private
   */
  _dispatchEvents (events) {
    this._log('dispatch events', events)

    for (const event of events) {
      document.dispatchEvent(new Event(event))
    }
  }

  /**
   * Logs debug output to the console.
   * @function
   * @private
   */
  _log () {
    if (this.options.debug) {
      console.debug('[CookieBar]', ...arguments)
    }
  }

  /**
   * Destroys an instance.
   * @function
   */
  _destroy () {
    $(window).off('.zf.cookieBar')
    this._hide()
    this._log('destroyed', this)
  }
}

/**
 * Default settings for plugin.
 */
CookieBar.defaults = {
  /**
   * The name under which to store the accepted events.
   * @option
   * @type {string}
   * @default 'cookieaccepted'
   */
  storeName: 'mo-cookiebar',

  /**
   * The event name to use if cookies for no specific event are accepted.
   * @option
   * @type {string}
   * @default 'tracking-allowed'
   */
  defaultEvent: 'tracking-allowed',

  /**
   * The CSS class name for elements opting in to tracking events when clicked.
   * @option
   * @type {string}
   * @default 'accept-cookies'
   */
  acceptClass: 'accept-cookies',

  /**
   * The CSS class name for elements blocking all tracking events when clicked.
   * @option
   * @type {string}
   * @default 'block-cookies'
   */
  blockClass: 'block-cookies',

  /**
   * Override the function for preparing the cookie bar display.
   * @option
   * @type {function}
   * @param {JQuery} $cookiebar - The cookiebar jQuery object.
   * @default 'null'
   */
  init: null,

  /**
   * Override the function for showing the cookie bar.
   * @option
   * @type {function}
   * @param {JQuery} $cookiebar - The cookiebar jQuery object.
   * @default 'null'
   */
  show: null,

  /**
   * Override the function for hiding the cookie bar.
   * @option
   * @type {function}
   * @param {JQuery} $cookiebar - The cookiebar jQuery object.
   * @default 'null'
   */
  hide: null,

  /**
   * Toggle debug output.
   * @option
   * @type {boolean}
   * @default false
   */
  debug: false
}

export { CookieBar }
