import $ from 'jquery'
import { Plugin } from 'foundation-sites/js/foundation.core.plugin'
import { GetYoDigits } from 'foundation-sites/js/foundation.core.utils'
import { Triggers } from 'foundation-sites/js/foundation.util.triggers'

/**
 * ToggleElement module.
 *
 * Plugin options can be set as individual data attributes, one combined data-options attribute,
 * or as an object passed to the plugin’s constructor. It is also possible to control a target
 * `data-toggle-element` with `data-toggle`, `data-open`, `data-close` on a controlling element.
 * In cases where the data attribute cannot be set on the target element, it may be added to the
 * controller element instead referencing the target’s id as its value.
 *
 * @module foundation.toggleElement
 */

class ToggleElement extends Plugin {
  /**
   * Creates a new instance of ToggleElement.
   * @class
   * @name ToggleElement
   * @fires ToggleElement#init
   * @param {JQuery} element - jQuery object on which the plugin has been initialized.
   * @param {Object} options - Overrides to the default plugin settings.
   */
  _setup (element, options) {
    this.$element = element
    this.$target = element
    this.$controller = null
    this.options = $.extend({}, ToggleElement.defaults, element.data(), options)
    this.className = 'ToggleElement'

    // Triggers init is idempotent, just need to make sure it is initialized
    Triggers.init($)

    this._init()
    this._events()
    this._log('setup', this)
  }

  /**
   * Initializes the ToggleElement plugin by parsing data-toggle-element attributes.
   * @function
   * @private
   */
  _init () {
    let $controller
    if (this.options.toggleElement) {
      this.$target = this._idToElement(this.options.toggleElement)
      $controller = this.$element
    } else if (this.options.controller) {
      $controller = this._idToElement(this.options.controller)
    }
    // Add event listeners for use cases not covered by Foundation.Triggers.
    if ($controller && this.options.action && this.options.event) {
      this.$controller = $controller
      this._addAria()
      const [ev, value] = this.options.event.trim().split(/ (.*)/, 2)
      $controller.on(`${ev}.zf.toggleElement`, (e) => {
        this._log('event', e)
        if (value === undefined || value === e.target.value) {
          // Trigger the same event as Foundation.Triggers would.
          this.$target.triggerHandler(`${this.options.action}.zf.trigger`, [$controller])
        }
      })
    }
  }

  /**
   * Helper function to convert a string of space separated ids into a jQuery element.
   * @function
   * @private
   * @param {string} id - A plain id name or more speparated by a space.
   * @return {JQuery} - A jQuery object matching the id.
   */
  _idToElement (id) {
    return $(id.split(' ').map(i => (`#${i}`)).join(', '))
  }

  /**
   * Adds ARIA attributes to the controller.
   *
   * For multiple targets, only the ID of the first item is referenced.
   * @function
   * @private
   */
  _addAria () {
    let targetId = this.$target.attr('id')
    if (!targetId) {
      targetId = GetYoDigits(6, 'toggle-element')
      this.$target.attr('id', targetId)
    }
    this.$controller.attr('aria-controls', targetId)
      .attr('aria-expanded', !this._isHidden(this.$target))
  }

  /**
   * Checks if the hidden class is set on the target.
   * @function
   * @private
   */
  _isHidden () {
    return this.$target.hasClass(this.options.hiddenClass)
  }

  /**
   * React to events.
   * @function
   * @private
   */
  _events () {
    this.$target.on('toggle.zf.trigger', (e, el) => {
      this._log('trigger toggle', el)
      this._isHidden() ? this.show(el) : this.hide(el)
    })
    this.$target.on('open.zf.trigger', (e, el) => {
      this._log('trigger open', el)
      if (this._isHidden()) {
        this.show(el)
      }
    })
    this.$target.on('close.zf.trigger', (e, el) => {
      this._log('trigger close', el)
      if (!this._isHidden()) {
        this.hide(el)
      }
    })
  }

  /**
   * Show the target.
   * @function
   * @fires ToggleElement#shown
   * @param {JQuery} $controller - The element that triggered this action.
   */
  show ($controller) {
    this._log('show', this.$target)
    this.$target.slideDown()
      .removeClass(this.options.hiddenClass)
    if ($controller) {
      $controller.attr('aria-expanded', true)
    }
    /**
     * Fires when the target is shown.
     * @event ToggleElement#shown
     */
    this.$target.trigger('shown.zf.toggleElement', [$controller])
  }

  /**
   * Hide the target.
   * @function
   * @fires ToggleElement#hidden
   * @param {JQuery} $controller - The element that triggered this action.
   */
  hide ($controller) {
    this._log('hide', this.$target)
    this.$target.addClass(this.options.hiddenClass)
      .slideUp()
    if ($controller) {
      $controller.attr('aria-expanded', false)
    }
    /**
     * Fires when the target is hidden.
     * @event ToggleElement#hidden
     */
    this.$target.trigger('hidden.zf.toggleElement', [$controller])
  }

  /**
   * Logs debug output to the console.
   * @function
   * @private
   */
  _log () {
    if (this.options.debug) {
      console.debug('[ToggleElement]', ...arguments)
    }
  }

  /**
   * Destroys the ToggleElement plugin.
   * @function
   */
  _destroy () {
    this._log('destroy', this)
    if (this.$controller) {
      this.$controller.off('.zf.toggleElement')
    }
  }
}

/**
 * Default settings for plugin.
 */
ToggleElement.defaults = {
  /**
   * The element that controls the target.
   * @option
   * @type {JQuery|null}
   * @default null
   */
  controller: null,

  /**
   * The action to apply to the element: 'open', 'close' or 'toggle'.
   * @option
   * @type {string}
   * @default 'toggle'
   */
  action: 'toggle',

  /**
   * The controller event to trigger the target being shown/hidden.
   *
   * If the event requires an additional value it can be added after a space,
   * e.g. 'change opt-in' if the controller is an input that should only
   * trigger when it has the value "opt-in" after a change event.
   * @option
   * @type {string}
   * @default 'click'
   */
  event: 'click',

  /**
   * The class set on the target while it’s hidden.
   *
   * If this is already there on page load, it will hide the target initially.
   * @option
   * @type {string}
   * @default 'toggle-element-hidden'
   */
  hiddenClass: 'toggle-element-hidden',

  /**
   * Toggle debug output.
   * @option
   * @type {boolean}
   * @default false
   */
  debug: false
}

export { ToggleElement }
